# Copyright 2019 (c) Anna Schumaker.
from .. import notify
from .. import sort
from .. import trak
from .. import tree
import random
import threading
from gi.repository import GLib

class Playlist(tree.ETree):
    def __init__(self, name, icon="", sort=[], can_loop=True, can_random=True):
        self.list       = [ ]
        self.sort_order = sort
        self.current    = -1
        self.can_loop   = can_loop
        self.loop       = False
        self.can_random = can_random
        self.random     = False
        self.visible    = False
        self.plist_lock = threading.Lock()
        tree.ETree.__init__(self, name, icon)

    def __find_track__(self, track):
        if track == None:
            return (False, None)
        if len(self.sort_order) == 0:
            try:
                return (True, self.list.index(track))
            except:
                return (False, len(self.list))
        key = track.sort_key(self.sort_order)
        (index, obj) = sort.bisect(self.list, key, self.sort_order)
        return (obj == track, index)

    def __getitem__(self, i):
        with self.plist_lock:
            return self.list[i] if i < len(self.list) else None

    def __getstate__(self):
        with self.plist_lock:
            state = tree.ETree.__getstate__(self)
            state["list"] = [ t.path for t in self.list ]
            del state["plist_lock"]
            return state

    def __len__(self):
        with self.plist_lock:
            return len(self.list)

    def __next__(self, cur):
        if (max := len(self.list) - 1) == -1:
            return None
        elif max == 0:
            return None if (self.loop == False and cur == 0) else 0
        elif self.random == True:
            return (cur + random.randint(1, max)) % max
        elif cur < max:
            return cur + 1
        elif self.loop == True:
            return 0
        return None

    def __setstate__(self, state):
        tree.ETree.__setstate__(self, state)
        self.list       = [ trak.lookup(path) for path in state["list"] ]
        self.sort_order = state["sort_order"]
        self.current    = state["current"]
        self.can_loop   = state["can_loop"]
        self.loop       = state["loop"]
        self.can_random = state["can_random"]
        self.random     = state["random"]
        self.visible    = False
        self.plist_lock = threading.Lock()

    def add(self, track):
        with self.plist_lock:
            (has, index) = self.__find_track__(track)
            if has == True or index == None:
                return
            self.list[index:index] = [ track ]
        self.track_added(track, index)

    def changed(self):
        notify.notify("playlist-changed", self)
        notify.notify("save-playlists")

    def contains(self, track):
        with self.plist_lock:
            return self.__find_track__(track)[0]

    def get_markup(self):
        t = GLib.markup_escape_text(self.name)
        l = len(self.list)
        return f"{t}\n{l} Track{'s' if l != 1 else ''}"

    def index(self, track):
        with self.plist_lock:
            (has, index) = self.__find_track__(track)
        return index if has == True else None

    def move(self, track, new):
        with self.plist_lock:
            (has, old) = self.__find_track__(track)
            if has == False:
                return

            self.list.pop(old)
            self.list.insert(new, track)
            self.sort_order.clear()

            if old == self.current:
                self.current = new
            elif new < old and new  <= self.current:
                self.current += 1
            elif new > old and new == self.current:
                self.current += 1
            elif new > old and new > self.current:
                self.current -= 1

    def next(self):
        with self.plist_lock:
            if (n := self.__next__(self.current)) != None:
                self.current = n
                return self.list[self.current]
            return None

    def peek(self, n):
        with self.plist_lock:
            cur = self.current
            res = [ ]

            for i in range(n):
                if (cur := self.__next__(cur)) == None:
                    break
                res.append(self.list[cur])
            return res

    def remove(self, track):
        with self.plist_lock:
            (has, pos) = self.__find_track__(track)
            if has == False:
                return
            track = self.list.pop(pos)
            if pos <= self.current:
                self.current -= 1
        self.track_removed(track, pos)

    def reset(self):
        with self.plist_lock:
            self.list.clear()
            self.current = -1
            self.loop    = False
            self.random  = False
            tree.ETree.reset(self)

    def runtime(self):
        with self.plist_lock:
            total = sum([ track.length for track in self.list ])
        m, s = divmod(total, 60)
        h, m = divmod(m, 60)
        d, h = divmod(h, 24)
        res  = [ ]
        if d > 0: res.append(f"{int(d)} day{'s' if d != 1 else ''}")
        if h > 0: res.append(f"{int(h)} hour{'s' if h != 1 else ''}")
        if m > 0: res.append(f"{int(m)} minute{'s' if m  != 1 else ''}")
        if s > 0: res.append(f"{int(s)} second{'s' if s  != 1 else ''}")
        return ", ".join(res)

    def set_loop(self, loop):
        if self.can_loop == True:
            self.loop = loop
        return self.loop

    def set_random(self, random):
        if self.can_random == True:
            self.random = random
        return self.random

    def sort(self, field):
        with self.plist_lock:
            if field in self.sort_order:
                self.sort_order.remove(field)
            else:
                self.sort_order.append(field)
            self.list.sort(key=self.track_key)
            return field in self.sort_order

    def track_added(self, track, index):
        if self.visible == True:
            notify.notify("add-track", self, track, index)
        self.changed()

    def track_key(self, track):
        return track.sort_key(self.sort_order)

    def track_removed(self, track, index):
        if self.visible == True:
            notify.notify("remove-track", self, track, index)
        self.changed()
