# Copyright 2019 (c) Anna Schumaker.
from . import autopause
from . import playbin
from .. import gtk
import curds
import os
import time
import unittest
from gi.repository import Gtk, Gst

test_album = os.path.abspath("./trier/Test Library/Test Artist 01/Test Album 1")
test_track = os.path.join(test_album, "10 - Test Track 10.ogg")

class TestAutoPause(unittest.TestCase):
    def setUp(self):
        curds.reset()
        playbin.reset()
        autopause.reset()

    def test_init(self):
        self.assertEqual(autopause.MINIMUM, -1)
        self.assertEqual(autopause.MAXIMUM, 99)

        self.assertIsInstance(autopause.Box,     Gtk.Box)
        self.assertIsInstance(autopause.Down,    Gtk.Button)
        self.assertIsInstance(autopause.Entry,   Gtk.Entry)
        self.assertIsInstance(autopause.Label,   Gtk.Label)
        self.assertIsInstance(autopause.Popover, Gtk.Popover)
        self.assertIsInstance(autopause.Up,      Gtk.Button)

        self.assertEqual(autopause.Count, autopause.MINIMUM)
        self.assertEqual(autopause.Icon,  Gtk.EntryIconPosition.PRIMARY)

        self.assertFalse(autopause.Playing)
        self.assertTrue( autopause.Up.is_sensitive())
        self.assertFalse(autopause.Down.is_sensitive())

    def test_show_more(self):
        self.assertFalse(autopause.Box.is_visible())

        curds.notify.notify("show-more", True)
        self.assertTrue(autopause.Box.is_visible())

        curds.notify.notify("show-more", False)
        self.assertFalse(autopause.Box.is_visible())

    def test_pause_up_clicked(self):
        autopause.Up.clicked()
        self.assertEqual(autopause.Count, 0)
        self.assertTrue(autopause.Up.is_sensitive())
        self.assertEqual(autopause.Entry.get_text(), "This Track")
        self.assertTrue( autopause.Popover.is_visible())

        autopause.Up.clicked()
        self.assertEqual(autopause.Count, 1)
        self.assertTrue(autopause.Up.is_sensitive())
        self.assertEqual(autopause.Entry.get_text(), "Next Track")

        autopause.Count = 99
        autopause.Up.clicked()
        self.assertEqual(autopause.Count, 99)
        self.assertFalse(autopause.Up.is_sensitive())
        self.assertEqual(autopause.Entry.get_text(), "99 Tracks")

    def test_pause_down_clicked(self):
        autopause.Count = 99
        autopause.Down.clicked()
        self.assertEqual(autopause.Count, 98)
        self.assertTrue(autopause.Down.is_sensitive())
        self.assertEqual(autopause.Entry.get_text(), "98 Tracks")
        self.assertTrue( autopause.Popover.is_visible())

        autopause.Down.clicked()
        self.assertEqual(autopause.Count, 97)
        self.assertTrue(autopause.Down.is_sensitive())
        self.assertEqual(autopause.Entry.get_text(), "97 Tracks")

        autopause.Count = -1
        autopause.Down.clicked()
        self.assertEqual(autopause.Count, -1)
        self.assertFalse(autopause.Down.is_sensitive())
        self.assertEqual(autopause.Entry.get_text(), "Paused")

    def test_pause_decrement(self):
        self.assertFalse(autopause.about_to_pause())

        autopause.Count = 2
        self.assertFalse(autopause.about_to_pause())
        self.assertEqual(autopause.decrement(), Gst.State.PLAYING)
        self.assertEqual(autopause.Count, 1)

        self.assertFalse(autopause.about_to_pause())
        self.assertEqual(autopause.decrement(), Gst.State.PLAYING)
        self.assertEqual(autopause.Count, 0)

        self.assertTrue( autopause.about_to_pause())
        self.assertEqual(autopause.decrement(), Gst.State.PAUSED)
        self.assertEqual(autopause.Count, -1)

        self.assertFalse(autopause.about_to_pause())
        self.assertEqual(autopause.decrement(), Gst.State.PLAYING)
        self.assertEqual(autopause.Count, -1)

    def test_pause_entry(self):
        autopause.Entry.set_text("3")
        autopause.Entry.activate()
        self.assertEqual(autopause.Count, 3)
        self.assertTrue( autopause.Popover.is_visible())

        autopause.Entry.set_text("100")
        autopause.Entry.activate()
        self.assertEqual(autopause.Count, 99)

        autopause.Entry.set_text("-5")
        autopause.Entry.activate()
        self.assertEqual(autopause.Count, -1)

    def test_pause_state_changed(self):
        autopause.Playing = False
        autopause.state_changed(Gst.State.PLAYING)
        self.assertEqual(autopause.Playing, True)
        self.assertEqual(autopause.Entry.get_text(), "Keep Playing")
        self.assertEqual(autopause.Entry.get_icon_name(autopause.Icon), "media-playback-start")

        autopause.state_changed(Gst.State.PAUSED)
        self.assertEqual(autopause.Playing, False)
        self.assertEqual(autopause.Entry.get_icon_name(autopause.Icon), "media-playback-pause")

    def test_pause_text(self):
        self.assertEqual(autopause.__text__(), "Paused")

        autopause.Playing = True
        self.assertEqual(autopause.__text__(), "Keep Playing")

        autopause.Count = 0
        self.assertEqual(autopause.__text__(), "This Track")

        autopause.Count = 1
        self.assertEqual(autopause.__text__(), "Next Track")

        autopause.Count = 2
        self.assertEqual(autopause.__text__(), "2 Tracks")

    def test_pause_icon(self):
        self.assertEqual(autopause.__icon__(), "media-playback-pause")

        autopause.Playing = True
        self.assertEqual(autopause.__icon__(), "media-playback-start")

        autopause.Count = 0
        self.assertEqual(autopause.__icon__(), "media-playback-pause")

    def test_pause_estimate(self):
        curds.playlist.lookup("Libraries").lookup(test_album)
        curds.playlist.library.join()

        playbin.set_uri(test_track, Gst.State.PAUSED)
        self.assertEqual(playbin.get_state(), Gst.State.PAUSED)

        autopause.Count = 0
        self.assertEqual(autopause.__estimate__(), "About 10 seconds")
        playbin.seek_step(6)
        self.assertEqual(playbin.get_state(), Gst.State.PAUSED)
        self.assertEqual(autopause.__estimate__(), "About 5 seconds")

        autopause.Count = 10
        self.assertEqual(autopause.__estimate__(), "About 1 minute")
        autopause.Count = 20
        self.assertEqual(autopause.__estimate__(), "About 1 minute 55 seconds")
        autopause.Count = 30
        self.assertEqual(autopause.__estimate__(), "About 2 minutes 50 seconds")

    def test_pause_popover(self):
        playbin.set_uri(test_track, Gst.State.PAUSED)
        self.assertEqual(playbin.get_state(), Gst.State.PAUSED)

        autopause.Popover.show()
        autopause.__popup__()
        gtk.main_loop(iteration_delay=0.02)
        self.assertFalse(autopause.Popover.is_visible())

        autopause.Count = 0
        autopause.__popup__()
        self.assertTrue(autopause.Popover.is_visible())
        self.assertEqual(autopause.Label.get_text(), "About 10 seconds")

        self.assertIsNotNone(autopause.PopCB)
        gtk.timeout_loop(5.75, iteration_delay=0.05)
        self.assertIsNone(autopause.PopCB)
        self.assertFalse(autopause.Popover.is_visible())

        autopause.__popup__()
        cb_id = autopause.PopCB
        autopause.__popup__()
        self.assertNotEqual(autopause.PopCB, cb_id)
