# Copyright 2019 (c) Anna Schumaker.
from . import autopause
from . import playbin
from . import widgets
from .. import gtk
import curds
import os
import unittest
from gi.repository import Gdk, Gtk, Gst

test_album = os.path.abspath("./trier/Test Library/Test Artist 01/Test Album 1")
test_track = os.path.join(test_album, "10 - Test Track 10.ogg")

class TestAudioWidgets(unittest.TestCase):
    def setUp(self):
        curds.reset()
        autopause.reset()
        playbin.reset()
        widgets.reset()

    def test_widgets_init(self):
        self.assertIsInstance(widgets.Artist,    Gtk.Label)
        self.assertIsInstance(widgets.Next,      Gtk.Button)
        self.assertIsInstance(widgets.Pause,     Gtk.Button)
        self.assertIsInstance(widgets.Play,      Gtk.Button)
        self.assertIsInstance(widgets.Position,  Gtk.Label)
        self.assertIsInstance(widgets.Previous,  Gtk.Button)
        self.assertIsInstance(widgets.Progress,  Gtk.Scale)
        self.assertIsInstance(widgets.Remaining, Gtk.Label)
        self.assertIsInstance(widgets.Title,     Gtk.Label)
        self.assertIsInstance(widgets.Volume,    Gtk.VolumeButton)

        self.assertFalse(widgets.Pause.is_visible())
        self.assertTrue( widgets.Play.is_visible())

        self.assertEqual(widgets.SEEK_STEP, 10)

    def test_volume_button(self):
        self.assertEqual(playbin.get_property("volume"), 1.0)
        self.assertEqual(widgets.Volume.get_value(), 100.0)

        widgets.Volume.set_value(50)
        self.assertEqual(playbin.get_property("volume"), 0.5)
        self.assertEqual(widgets.Volume.get_value(), 50.0)

        widgets.Volume.set_value(0)
        self.assertEqual(playbin.get_property("volume"), 0.0)
        self.assertEqual(widgets.Volume.get_value(), 0.0)

    def test_play_pause_buttons(self):
        playbin.set_uri(test_track)

        widgets.Play.clicked()
        self.assertEqual(playbin.get_state(), Gst.State.PLAYING)
        gtk.main_loop()
        self.assertFalse(widgets.Play.is_visible())
        self.assertTrue( widgets.Pause.is_visible())
        self.assertTrue( autopause.Playing)

        widgets.Pause.clicked()
        self.assertEqual(playbin.get_state(), Gst.State.PAUSED)
        gtk.main_loop()
        self.assertTrue( widgets.Play.is_visible())
        self.assertFalse(widgets.Pause.is_visible())
        self.assertFalse( autopause.Playing)

    def test_next_previous_buttons(self):
        widgets.Next.clicked()
        self.assertEqual(playbin.get_state(), Gst.State.NULL)
        self.assertEqual(curds.playlist.Track, None)

        widgets.Previous.clicked()
        self.assertEqual(playbin.get_state(), Gst.State.NULL)
        self.assertEqual(curds.playlist.Track, None)

        curds.playlist.lookup("Libraries").lookup(test_album)
        curds.playlist.library.join()
        gtk.notify_loop()

        widgets.Next.clicked()
        self.assertEqual(playbin.get_state(), Gst.State.PLAYING)
        self.assertNotEqual(curds.playlist.Track, None)
        track = curds.playlist.Track

        widgets.Next.clicked()
        gtk.main_loop(delay=0.1)
        self.assertEqual(playbin.get_state(), Gst.State.PLAYING)
        self.assertNotEqual(curds.playlist.Track, track)
        track2 = curds.playlist.Track

        widgets.Previous.clicked()
        self.assertEqual(playbin.get_state(), Gst.State.PLAYING)
        self.assertNotEqual(curds.playlist.Track, track2)

        track3 = curds.playlist.next()
        widgets.NEXT_TRACK = track3
        widgets.Next.clicked()
        gtk.main_loop(delay=0.2)
        self.assertEqual(playbin.get_state(), Gst.State.PLAYING)
        self.assertEqual(curds.playlist.Track, track3)

    def test_seek_scale(self):
        playbin.set_uri(test_track, Gst.State.PAUSED)
        self.assertEqual(playbin.get_state(), Gst.State.PAUSED)
        self.assertEqual(playbin.position(),  0.0)

        widgets.Progress.emit("change-value", Gtk.ScrollType.JUMP, 50)
        self.assertEqual(playbin.get_state(), Gst.State.PAUSED)
        self.assertEqual(playbin.position(), 5 * Gst.SECOND)

    def test_seek_step(self):
        widgets.SEEK_STEP = 5
        event = Gdk.EventKey.new(Gdk.EventType.KEY_PRESS)

        playbin.set_uri(test_track, Gst.State.PAUSED)
        self.assertEqual(playbin.get_state(), Gst.State.PAUSED)
        self.assertEqual(playbin.position(),  0.0)

        event.keyval = Gdk.keyval_from_name("Right")
        gtk.Window.emit("key-press-event", event)
        self.assertEqual(playbin.get_state(), Gst.State.PAUSED)
        self.assertEqual(playbin.position(),  0.0)

        event.state |= Gdk.ModifierType.CONTROL_MASK
        gtk.Window.emit("key-press-event", event)
        self.assertEqual(playbin.get_state(), Gst.State.PAUSED)
        self.assertEqual(playbin.position(),  5 * Gst.SECOND)

        widgets.SEEK_STEP = 2
        event.keyval = Gdk.keyval_from_name("Left")
        gtk.Window.emit("key-press-event", event)
        self.assertEqual(playbin.get_state(), Gst.State.PAUSED)
        self.assertEqual(playbin.position(),  3 * Gst.SECOND)

    def test_timestr(self):
        self.assertEqual(widgets.timestr(   2 * Gst.SECOND), "00:02")
        self.assertEqual(widgets.timestr(  42 * Gst.SECOND), "00:42")
        self.assertEqual(widgets.timestr( 162 * Gst.SECOND), "02:42")
        self.assertEqual(widgets.timestr(2562 * Gst.SECOND), "42:42")

    def test_position(self):
        playbin.set_uri(test_track, Gst.State.PAUSED)
        self.assertEqual(playbin.get_state(), Gst.State.PAUSED)

        gtk.main_loop(delay=0.1)
        self.assertEqual(widgets.Position.get_text(),  "00:00")
        self.assertEqual(widgets.Remaining.get_text(), "00:10")
        self.assertEqual(widgets.Progress.get_adjustment().get_value(), 0.0)

        playbin.seek_step(3)
        self.assertEqual(playbin.get_state(), Gst.State.PAUSED)
        gtk.main_loop(delay=0.1)

        self.assertEqual(widgets.Position.get_text(),  "00:03")
        self.assertEqual(widgets.Remaining.get_text(), "00:07")
        self.assertEqual(widgets.Progress.get_adjustment().get_value(), 30.0)

    def test_tags(self):
        self.assertEqual(widgets.Title.get_text(), "Emmental")
        self.assertEqual(widgets.Artist.get_text(), "The Cheesy Music Player")

        playbin.set_uri(test_track, Gst.State.PAUSED)
        self.assertEqual(playbin.get_state(), Gst.State.PAUSED)
        gtk.main_loop()

        self.assertEqual(widgets.Title.get_text(), "Test Track 10")
        self.assertEqual(widgets.Artist.get_text(), "by Test Artist 01")

        playbin.Playbin.set_state(Gst.State.READY)
        self.assertEqual(playbin.get_state(), Gst.State.READY)
        gtk.main_loop()

        self.assertEqual(widgets.Title.get_text(), "Emmental")
        self.assertEqual(widgets.Artist.get_text(), "The Cheesy Music Player")

    def on_stream_start(self):
        self.stream_start += 1

    def test_stream_start(self):
        self.stream_start  = 0
        widgets.NEXT_TRACK = 42
        curds.notify.register("stream-start", self.on_stream_start)

        playbin.set_uri(test_track, Gst.State.PLAYING)
        gtk.main_loop(delay=0.1)
        self.assertEqual(self.stream_start, 1)
        self.assertIsNone(widgets.NEXT_TRACK)

    def test_eos(self):
        curds.playlist.lookup("Libraries").lookup(test_album)
        curds.playlist.library.join()

        track = curds.playlist.next()
        widgets.on_message(playbin.Bus, Gst.Message.new_eos())
        self.assertEqual(playbin.get_state(), Gst.State.PLAYING)
        self.assertNotEqual(curds.playlist.Track, track)

        track = curds.playlist.Track
        autopause.Count = 0
        widgets.on_message(playbin.Bus, Gst.Message.new_eos())
        self.assertEqual(autopause.Count, -1)
        self.assertEqual(playbin.get_state(), Gst.State.PAUSED)
        self.assertNotEqual(curds.playlist.Track, track)

    def test_about_to_finish(self):
        curds.playlist.lookup("Libraries").lookup(test_album)
        curds.playlist.library.join()

        track = curds.playlist.next()
        playbin.set_uri(track.path, Gst.State.PLAYING)

        widgets.on_about_to_finish(playbin.Playbin)
        self.assertNotEqual( widgets.NEXT_TRACK, track)
        self.assertIsNotNone(widgets.NEXT_TRACK)

        track = widgets.NEXT_TRACK
        autopause.Count = 1
        widgets.on_about_to_finish(playbin.Playbin)
        self.assertEqual(autopause.Count, 0)
        self.assertNotEqual( widgets.NEXT_TRACK, track)
        self.assertIsNotNone(widgets.NEXT_TRACK)

        track = widgets.NEXT_TRACK
        widgets.NEXT_TRACK = None
        widgets.on_about_to_finish(playbin.Playbin)
        self.assertEqual(autopause.Count, 0)
        self.assertIsNone(widgets.NEXT_TRACK)
