# Copyright 2019 (c) Anna Schumaker.
from . import model
from .. import gtk
from .. import playlist
import curds
import os
import time
import unittest
from gi.repository import Gtk, GObject, GLib

test_album1 = os.path.abspath("./trier/Test Library/Test Artist 02/Test Album 1")
test_album2 = os.path.abspath("./trier/Test Library/Test Artist 02/Test Album 2")

class TestNodeTreeModel(unittest.TestCase):
    def setUp(self):
        curds.reset()

        self.model  = model.NodeTreeModel()
        self.artist = curds.playlist.lookup("Artists")
        self.decade = curds.playlist.lookup("Decades")
        self.genre  = curds.playlist.lookup("Genres")
        curds.trak.lookup(os.path.join(test_album1, "01 - Test Track 01.ogg"))
        curds.trak.lookup(os.path.join(test_album2, "01 - Test Track 01.ogg"))

    def tearDown(self):
        curds.notify.cancel("first-child",      self.model.on_first_child)
        curds.notify.cancel("playlist-changed", self.model.on_playlist_changed)

    def tearDownClass():
        curds.stop()

    def test_model_init(self):
        self.assertEqual(model.Treeview, gtk.Builder.get_object("node_treeview"))

        self.assertEqual(self.model.get_n_columns(), 3)
        self.assertEqual(self.model.get_flags(), 0)

        self.assertEqual(self.model.get_column_type(0), GObject.GType(str))
        self.assertEqual(self.model.get_column_type(1), GObject.GType(str))
        self.assertEqual(self.model.get_column_type(2), GObject.GType(int))

    def test_model_get_iter(self):
        path = Gtk.TreePath.new_from_indices([ curds.playlist.Root.n_children() ])
        self.assertRaises(ValueError, self.model.get_iter, path)

        for node in list(curds.playlist.Root.walk()):
            path = Gtk.TreePath(node.get_path())
            iter = self.model.get_iter(path)
            self.assertEqual(iter.user_data, id(node))

    def test_model_get_path(self):
        iter = Gtk.TreeIter()

        for node in list(curds.playlist.Root.walk()):
            iter.user_data = id(node)
            path = self.model.get_path(iter)
            self.assertEqual(path.get_indices(), node.get_path())

    def test_model_get_value(self):
        col  = model.Treeview.get_column(1)
        rect = model.Treeview.get_cell_area(Gtk.TreePath(0), col)
        iter = Gtk.TreeIter()

        for child in curds.playlist.Root.walk():
            iter.user_data = id(child)
            text   = child.get_markup()
            height = rect.height
            if child == curds.playlist.Current[0]:
                text = f"<b>{text}</b>"
            if isinstance(child, curds.Playlist):
                height = -1

            self.assertEqual(self.model.get_value(iter, 0), child.icon)
            self.assertEqual(self.model.get_value(iter, 1), text)
            self.assertEqual(self.model.get_value(iter, 2), height)

    def test_model_iter_children(self):
        iter = self.model.iter_children(None)
        self.assertEqual(iter.user_data, id(curds.playlist.Root.nth_child(0)))

        for node in list(curds.playlist.Root.walk()):
            iter.user_data = id(node)
            if node.n_children() == 0:
                self.assertIsNone(self.model.iter_children(iter))
            else:
                child = self.model.iter_children(iter)
                self.assertEqual(child.user_data, id(node.nth_child(0)))

        iter.user_data = 42
        self.assertIsNone(self.model.iter_children(iter))

    def test_model_iter_has_child(self):
        iter = Gtk.TreeIter()

        for node in list(curds.playlist.Root.walk()):
            iter.user_data = id(node)
            if node.n_children() > 0:
                self.assertTrue(self.model.iter_has_child(iter))
            else:
                self.assertFalse(self.model.iter_has_child(iter))

        iter.user_data = 42
        self.assertFalse(self.model.iter_has_child(iter))

    def test_model_iter_n_children(self):
        iter = Gtk.TreeIter()
        self.assertEqual(self.model.iter_n_children(None), curds.playlist.Root.n_children())

        for node in list(curds.playlist.Root.walk()):
            iter.user_data = id(node)
            self.assertEqual(self.model.iter_n_children(iter), node.n_children())

        iter.user_data = 42
        self.assertEqual(self.model.iter_n_children(iter), 0)

    def test_model_iter_nth_child(self):
        iter = Gtk.TreeIter()

        for node in [ curds.playlist.Root ] + list(curds.playlist.Root.walk()):
            iter = self.model.playlist_iter(node)
            for i, child in enumerate(node.children):
                citer = self.model.iter_nth_child(iter, i)
                self.assertEqual(citer.user_data, id(child))
            self.assertIsNone(self.model.iter_nth_child(iter, node.n_children()))

    def test_model_iter_next(self):
        iter = Gtk.TreeIter()

        for node in list(curds.playlist.Root.walk()):
            iter.user_data = id(node)
            next = self.model.iter_next(iter)
            if node.next_child() != None:
                self.assertEqual(next.user_data, id(node.next_child()))
            else:
                self.assertIsNone(next)

    def test_model_iter_parent(self):
        iter = Gtk.TreeIter()

        for node in curds.playlist.Root.walk():
            iter.user_data = id(node)
            if node.parent != curds.playlist.Root:
                parent = self.model.iter_parent(iter)
                self.assertEqual(parent.user_data, id(node.parent))
            else:
                self.assertIsNone(self.model.iter_parent(iter))

    def test_model_iter_playlist(self):
        iter = Gtk.TreeIter()

        self.assertEqual(self.model.iter_playlist(None), curds.playlist.Root)
        self.assertEqual(self.model.iter_playlist(iter), curds.playlist.Root)

        for node in curds.playlist.Root.walk():
            iter.user_data = id(node)
            self.assertEqual(self.model.iter_playlist(iter), node)

        iter.user_data = 42
        self.assertIsNone(self.model.iter_playlist(iter))

    def test_model_playlist_iter(self):
        self.assertIsNone(self.model.playlist_iter(None))
        self.assertIsNone(self.model.playlist_iter(curds.playlist.Root))

        for node in curds.playlist.Root.walk():
            iter = self.model.playlist_iter(node)
            self.assertEqual(iter.user_data, id(node))
