# Copyright 2019 (c) Anna Schumaker.
from . import model
from . import view
from .. import gtk
from .. import playlist
import curds
import os
import re
import unittest
from gi.repository import Gtk, GObject

class TestNodeTreeView(unittest.TestCase):
    def setUpClass():
        curds.reset()
        view.reset()

    def tearDownClass():
        view.reset()

    def test_init(self):
        self.assertIsInstance(view.Entry,     Gtk.SearchEntry)
        self.assertIsInstance(view.Error,     Gtk.Label)
        self.assertIsInstance(view.Filter,    Gtk.TreeModelFilter)
        self.assertIsInstance(view.Model,     model.NodeTreeModel)
        self.assertIsInstance(view.Pattern,   re.Pattern)
        self.assertIsInstance(view.Popover,   Gtk.Popover)
        self.assertIsInstance(view.TreeView,  Gtk.TreeView)
        self.assertIsInstance(view.Selection, Gtk.TreeSelection)
        self.assertIsInstance(view.Separator, Gtk.Separator)

        self.assertEqual(view.TreeView.get_model(), view.Filter)
        self.assertEqual(view.Pattern.pattern, "")

    def test_show_more_accel(self):
        signal = GObject.signal_lookup("grab-focus", Gtk.SearchEntry)

        self.assertFalse(view.Entry.is_visible())
        self.assertTrue( view.Entry.can_activate_accel(signal))
        self.assertTrue( view.Entry.is_visible())
        self.assertTrue( view.Separator.is_visible())

        view.Entry.grab_focus()
        self.assertFalse(view.Entry.can_activate_accel(signal))
        self.assertFalse(view.Entry.is_visible())
        self.assertFalse(view.Separator.is_visible())

    def test_show_more_notification(self):
        curds.notify.notify("show-more", True)
        self.assertTrue(view.Entry.is_visible())
        self.assertTrue(view.Separator.is_visible())

        curds.notify.notify("show-more", False)
        self.assertFalse(view.Entry.is_visible())
        self.assertFalse(view.Separator.is_visible())

    def test_child_inserted(self):
        path    = os.path.abspath("./trier/Test Library/Test Artist 01")
        library = curds.playlist.lookup("Libraries")
        user    = curds.playlist.lookup("Playlists")
        row     = Gtk.TreePath(library.get_path())

        crow = Gtk.TreePath(user.get_path())
        row  = view.Filter.convert_child_path_to_path(crow)
        view.TreeView.collapse_row(row)

        for child in curds.playlist.Root.children:
            crow = Gtk.TreePath(child.get_path())
            row  = view.Filter.convert_child_path_to_path(crow)
            self.assertFalse(view.TreeView.row_expanded(row))

        library.lookup(path)
        curds.playlist.library.join()
        gtk.main_loop()

        for child in curds.playlist.Root.children:
            crow = Gtk.TreePath(child.get_path())
            row  = view.Filter.convert_child_path_to_path(crow)
            if child == library:
                self.assertTrue(view.TreeView.row_expanded(row))
            else:
                self.assertFalse(view.TreeView.row_expanded(row))

    def test_can_select_path(self):
        root = curds.playlist.Root
        view.TreeView.expand_all()

        for child in list(root.walk()):
            path = Gtk.TreePath(child.get_path())
            view.Selection.select_path(path)
            if isinstance(child, curds.Playlist):
                self.assertTrue(view.Selection.path_is_selected(path))
            else:
                self.assertFalse(view.Selection.path_is_selected(path))

    def test_select_path(self):
        collection = curds.playlist.lookup("Collection")
        previous   = curds.playlist.lookup("Previous")

        self.assertNotEqual(playlist.view.Model.playlist, previous)

        path = Gtk.TreePath(previous.get_path())
        view.Selection.select_path(path)
        self.assertEqual(playlist.view.Model.playlist, previous)

        path = Gtk.TreePath(collection.get_path())
        view.Selection.select_path(path)
        self.assertEqual(playlist.view.Model.playlist, collection)

        view.Selection.unselect_all()
        self.assertEqual(playlist.view.Model.playlist, collection)

    def test_row_activate(self):
        collection = curds.playlist.lookup("Collection")
        playlists  = curds.playlist.lookup("Playlists")
        column     = view.TreeView.get_column(1)

        self.assertEqual(curds.playlist.current(), collection)

        path = Gtk.TreePath(curds.playlist.Starred.get_path())
        path = view.Filter.convert_child_path_to_path(path)
        view.TreeView.row_activated(path, column)
        self.assertEqual(curds.playlist.current(), curds.playlist.Starred)

        path = Gtk.TreePath(playlists.get_path())
        path = view.Filter.convert_child_path_to_path(path)
        view.TreeView.row_activated(path, column)
        self.assertEqual(curds.playlist.current(), curds.playlist.Starred)

    def test_filter_error(self):
        view.Entry.set_text("*v")
        view.Entry.emit("search-changed")

        self.assertTrue(view.Entry.get_style_context().has_class("warning"))
        self.assertTrue(view.Popover.is_visible())
        self.assertEqual(view.Error.get_text(), "Nothing to repeat at position 0")

        view.Entry.set_text("")
        view.Entry.emit("search-changed")
        gtk.main_loop(iteration_delay=0.01)

        self.assertFalse(view.Entry.get_style_context().has_class("warning"))
        self.assertFalse(view.Popover.is_visible())

    def test_filter_nodes(self):
        root = curds.playlist.Root
        user = root.lookup("Playlists")
        star = user.lookup("Starred")
        stär = curds.Playlist("Stärred (Semi)", "semi-starred")
        iter = None

        user.insert_child(stär)
        self.assertEqual(user.n_children(), 3)

        self.assertEqual(view.Filter.iter_n_children(None), root.n_children())
        view.Entry.set_text("star")
        view.Entry.emit("search-changed")

        self.assertEqual(view.Filter.iter_n_children(iter), 1)
        iter  = view.Filter.iter_nth_child(iter, 0)
        child = view.Filter.convert_iter_to_child_iter(iter)
        self.assertEqual(view.Model.iter_playlist(child), user)

        self.assertEqual(view.Filter.iter_n_children(iter), 2)
        iter  = view.Filter.iter_nth_child(iter, 0)
        child = view.Filter.convert_iter_to_child_iter(iter)
        self.assertEqual(view.Model.iter_playlist(child), star)

        iter  = view.Filter.iter_next(iter)
        child = view.Filter.convert_iter_to_child_iter(iter)
        self.assertEqual(view.Model.iter_playlist(child), stär) 

        view.Entry.set_text("")
        view.Entry.emit("search-changed")
        self.assertEqual(view.Filter.iter_n_children(None), root.n_children())
