# Copyright 2019 (c) Anna Schumaker.
from . import model
import curds
import os
import unittest
from gi.repository import Gtk, GObject

test_album = os.path.abspath("./trier/Test Library/Test Artist 01/Test Album 1")

class TestPlaylistModel(unittest.TestCase):
    def setUp(self):
        curds.reset()

        self.playlist = curds.Playlist("Test Playlist")
        self.track1 = curds.trak.lookup(os.path.join(test_album, "01 - Test Track 01.ogg"))
        self.track2 = curds.trak.lookup(os.path.join(test_album, "02 - Test Track 02.ogg"))
        self.playlist.add(self.track1)
        self.playlist.add(self.track2)

        self.model = model.PlaylistModel(self.playlist)

    def test_model_init(self):
        self.assertEqual(self.model.playlist, self.playlist)
        self.assertTrue( self.model.playlist.visible)

        self.assertEqual(self.model.get_n_columns(), 10)
        self.assertEqual(self.model.get_flags(), Gtk.TreeModelFlags.LIST_ONLY)
        for col in range(self.model.get_n_columns()):
            self.assertEqual(self.model.get_column_type(col), GObject.GType(str))

    def test_model_del(self):
        plist = curds.Playlist("Test Playlist Two")
        self.assertTrue( self.playlist.visible)
        self.model = model.PlaylistModel(plist)
        self.assertTrue(plist.visible)
        self.assertFalse(self.playlist.visible)

    def test_model_drag_data_delete(self):
        self.assertTrue( self.model.drag_data_delete(Gtk.TreePath(0)))
        self.assertEqual(len(self.model), 2)
        self.assertTrue( self.model.drag_data_delete(Gtk.TreePath(1)))
        self.assertEqual(len(self.model), 2)
        self.assertFalse(self.model.drag_data_delete(Gtk.TreePath(2)))

    def test_model_drag_data_get(self):
        # There is currently no way to manually allocate the
        # Gtk.SelectionData that we need to test this function
        pass

    def on_row_changed(self, model, path, iter):
        self.row_changed += 1

    def test_model_drag_data_received(self):
        cb = self.model.connect("row-changed", self.on_row_changed)
        self.row_changed = 0

        self.model.move_track(Gtk.TreePath(0), Gtk.TreePath(1))
        self.assertEqual(self.playlist[0], self.track2)
        self.assertEqual(self.playlist[1], self.track1)
        self.assertEqual(self.row_changed, 2)

        self.model.move_track(Gtk.TreePath(1), Gtk.TreePath(0))
        self.assertEqual(self.playlist[0], self.track1)
        self.assertEqual(self.playlist[1], self.track2)
        self.assertEqual(self.row_changed, 4)

        self.model.disconnect(cb)

    def test_model_get_iter(self):
        iter = self.model.get_iter(Gtk.TreePath(0))
        self.assertEqual(iter.user_data, 0)
        iter = self.model.get_iter(Gtk.TreePath(1))
        self.assertEqual(iter.user_data, 1)
        self.assertRaises(ValueError, self.model.get_iter, Gtk.TreePath(2))

    def test_model_get_path(self):
        iter = Gtk.TreeIter()
        iter.user_data = 0
        self.assertEqual(self.model.get_path(iter), Gtk.TreePath(0))
        iter.user_data = 1
        self.assertEqual(self.model.get_path(iter), Gtk.TreePath(1))

    def test_model_get_value(self):
        iter = self.model.track_iter(self.track1)
        self.assertEqual(self.model.get_value(iter, 0), "non-starred")
        self.assertEqual(self.model.get_value(iter, 1), "edit-redo-symbolic")
        self.assertEqual(self.model.get_value(iter, 2), "01")
        self.assertEqual(self.model.get_value(iter, 3), "Test Track 01")
        self.assertEqual(self.model.get_value(iter, 4), "0:01")
        self.assertEqual(self.model.get_value(iter, 5), "Test Artist 01")
        self.assertEqual(self.model.get_value(iter, 6), "Test Album 1")
        self.assertEqual(self.model.get_value(iter, 7), "Disc 1")
        self.assertEqual(self.model.get_value(iter, 8), "1973")
        self.assertEqual(self.model.get_value(iter, 9).title(), "Test Genre 1")

        iter = self.model.track_iter(self.track2)
        curds.playlist.Starred.add(self.track2)
        curds.playlist.UpNext.add(self.track2)

        self.assertEqual(self.model.get_value(iter, 0), "starred")
        self.assertEqual(self.model.get_value(iter, 1), "edit-redo")
        self.assertEqual(self.model.get_value(iter, 2), "02")
        self.assertEqual(self.model.get_value(iter, 3), "Test Track 02")
        self.assertEqual(self.model.get_value(iter, 4), "0:02")
        self.assertEqual(self.model.get_value(iter, 5), "Test Artist 01")
        self.assertEqual(self.model.get_value(iter, 6), "Test Album 1")
        self.assertEqual(self.model.get_value(iter, 7), "Disc 1")
        self.assertEqual(self.model.get_value(iter, 8), "1973")
        self.assertEqual(self.model.get_value(iter, 9).title(), "Test Genre 1")

    def test_model_iter_children(self):
        iter = self.model.track_iter(self.track1)
        self.assertIsNone(self.model.iter_children(iter))

    def test_model_has_child(self):
        iter = self.model.track_iter(self.track1)
        self.assertFalse(self.model.iter_has_child(iter))

    def test_model_n_children(self):
        iter = self.model.track_iter(self.track1)
        self.assertEqual(self.model.iter_n_children(None), 2)
        self.assertEqual(self.model.iter_n_children(iter), 0)

    def test_model_iter_nth_child(self):
        iter = self.model.iter_nth_child(None, 0)
        self.assertEqual(iter.user_data, 0)
        iter = self.model.iter_nth_child(None, 1)
        self.assertEqual(iter.user_data, 1)
        self.assertIsNone(self.model.iter_nth_child(None, 2))
        self.assertIsNone(self.model.iter_nth_child(iter, 0))

    def test_model_iter_next(self):
        iter = self.model.iter_nth_child(None, 0)
        self.assertEqual(iter.user_data, 0)
        iter = self.model.iter_next(iter)
        self.assertEqual(iter.user_data, 1)
        self.assertIsNone(self.model.iter_next(iter))

    def test_model_row_draggable(self):
        self.assertTrue( self.model.row_draggable(Gtk.TreePath(0)))
        self.assertTrue( self.model.row_draggable(Gtk.TreePath(1)))
        self.assertFalse(self.model.row_draggable(Gtk.TreePath(2)))

    def test_model_row_drop_possible(self):
        # There is currently no way to manually allocate the
        # Gtk.SelectionData that we need to test this function
        pass

    def test_model_iter_track(self):
        iter = Gtk.TreeIter()
        iter.user_data = 0
        self.assertEqual(self.model.iter_track(iter), self.track1)
        iter.user_data = 1
        self.assertEqual(self.model.iter_track(iter), self.track2)
        iter.user_data = 2
        self.assertEqual(self.model.iter_track(iter), None)

    def test_model_path_track(self):
        self.assertEqual(self.model.path_track(Gtk.TreePath(0)), self.track1)
        self.assertEqual(self.model.path_track(Gtk.TreePath(1)), self.track2)
        self.assertEqual(self.model.path_track(Gtk.TreePath(2)), None)

    def test_model_track_iter(self):
        iter = self.model.track_iter(self.track1)
        self.assertEqual(iter.user_data, 0)
        iter = self.model.track_iter(self.track2)
        self.assertEqual(iter.user_data, 1)
        self.assertEqual(self.model.track_iter(None), None)
