# Copyright 2019 (c) Anna Schumaker.
from . import model
from . import view
from .. import audio
from .. import gtk
import curds
import os
import re
import unittest
from gi.repository import Gdk, GObject, Gtk, Gst

test_album = os.path.abspath("./trier/Test Library/Test Artist 01/Test Album 1")

class TestPlaylistView(unittest.TestCase):
    def setUp(self):
        curds.reset()
        audio.reset()
        view.reset()

        self.plist = curds.Playlist("Test Playlist")
        view.switch(self.plist)

    def tearDownClass():
        view.TreeView.set_model(None)

    def add_track(self, trackno):
        path  = os.path.join(test_album, f"{trackno:02} - Test Track {trackno:02}.ogg")
        track = curds.trak.lookup(path)
        self.plist.add(track)
        gtk.notify_loop()

    def test_init(self):
        self.assertIsInstance(view.Entry,     Gtk.Entry)
        self.assertIsInstance(view.Error,     Gtk.Label)
        self.assertIsInstance(view.Filter,    Gtk.TreeModelFilter)
        self.assertIsInstance(view.Loop,      Gtk.ToggleButton)
        self.assertIsInstance(view.Model,     model.PlaylistModel)
        self.assertIsInstance(view.Pattern,   re.Pattern)
        self.assertIsInstance(view.Popover,   Gtk.Popover)
        self.assertIsInstance(view.Random,    Gtk.ToggleButton)
        self.assertIsInstance(view.TreeView,  Gtk.TreeView)
        self.assertIsInstance(view.Selection, Gtk.TreeSelection)
        self.assertIsInstance(view.Separator, Gtk.Separator)

        view.reset()
        self.assertEqual(view.Model.playlist, curds.playlist.lookup("Collection"))
        self.assertEqual(view.NextPos, 0)
        self.assertEqual(view.TreeView.get_model(), view.Filter)
        self.assertEqual(view.Pattern.pattern, "")

    def test_show_more_accel(self):
        signal = GObject.signal_lookup("grab-focus", Gtk.SearchEntry)

        self.assertFalse(view.Entry.is_visible())
        self.assertTrue( view.Entry.can_activate_accel(signal))
        self.assertTrue( view.Entry.is_visible())
        self.assertTrue( view.Separator.is_visible())

        view.Entry.grab_focus()
        self.assertFalse(view.Entry.can_activate_accel(signal))
        self.assertFalse(view.Entry.is_visible())
        self.assertFalse(view.Separator.is_visible())

    def test_show_more_notification(self):
        curds.notify.notify("show-more", True)
        self.assertTrue(view.Entry.is_visible())
        self.assertTrue(view.Separator.is_visible())

        curds.notify.notify("show-more", False)
        self.assertFalse(view.Entry.is_visible())
        self.assertFalse(view.Separator.is_visible())

    def test_switch(self):
        plist = curds.Playlist("Test Playlist 2")
        view.switch(plist)
        self.assertEqual(view.Model.playlist, plist)
        self.assertEqual(view.Filter.get_model(), view.Model)

        view.NextPos = 42
        model = view.Model
        view.switch(plist)
        self.assertEqual(view.Model, model)

        view.switch(plist, force=True)
        self.assertNotEqual(view.Model, model)
        self.assertEqual(view.NextPos, 0)

    def test_loop_toggled(self):
        self.assertFalse(self.plist.loop)
        view.Loop.set_active(True)
        self.assertTrue(self.plist.loop)
        view.Loop.set_active(False)
        self.assertFalse(self.plist.loop)

        self.plist.can_loop = False
        view.Loop.set_active(True)
        self.assertFalse(self.plist.loop)
        self.assertFalse(view.Loop.get_active())

    def test_random_toggled(self):
        self.assertFalse(self.plist.random)
        view.Random.set_active(True)
        self.assertTrue(self.plist.random)
        view.Random.set_active(False)
        self.assertFalse(self.plist.random)

        self.plist.can_random = False
        view.Random.set_active(True)
        self.assertFalse(self.plist.random)
        self.assertFalse(view.Random.get_active())

    def test_switch_toggle_buttons(self):
        self.plist.can_random = False
        self.plist.can_loop = False
        view.switch(self.plist, force=True)
        self.assertFalse(view.Loop.is_sensitive())
        self.assertFalse(view.Random.is_sensitive())

        self.plist.can_random = True
        self.plist.can_loop = True
        self.plist.random = True
        self.plist.loop = True
        view.switch(self.plist, force=True)
        self.assertTrue(view.Loop.is_sensitive())
        self.assertTrue(view.Loop.get_active())
        self.assertTrue(view.Random.is_sensitive())
        self.assertTrue(view.Random.get_active())

    def test_column_clicks(self):
        self.plist.sort_order = [ ]
        self.assertFalse(view.TreeView.get_columns()[0].get_clickable())

        for i, col in enumerate(view.TreeView.get_columns()[2:]):
            self.assertTrue(col.get_clickable())

            col.clicked()
            self.assertEqual(self.plist.sort_order, [ model.cols[i+2] ])
            self.assertEqual(col.get_sort_indicator(), True)
            self.assertEqual(col.get_sort_order(), Gtk.SortType.ASCENDING)

            col.clicked()
            self.assertEqual(self.plist.sort_order, [ ])
            self.assertEqual(col.get_sort_order(), False)

    def on_row_inserted(self, model, path, iter):
        self.row_inserted += 1

    def on_row_deleted(self, model, path):
        self.row_deleted += 1

    def test_add_remove_track(self):
        cb1 = view.Model.connect("row-inserted", self.on_row_inserted)
        cb2 = view.Model.connect("row-deleted",  self.on_row_deleted)
        self.row_inserted = 0
        self.row_deleted  = 0

        for i in range(1, 11):
            track = curds.trak.lookup(os.path.join(test_album, f"{i:02} - Test Track {i:02}.ogg"))
            self.plist.add(track)
            gtk.notify_loop()
            self.assertEqual(self.row_inserted, i)
            self.assertEqual(view.NextPos, i)

        for i in range(1, 11):
            self.plist.remove(self.plist[0])
            gtk.notify_loop()
            self.assertEqual(self.row_deleted, i)
            self.assertEqual(view.NextPos, len(self.plist))

        view.Model.disconnect(cb1)
        view.Model.disconnect(cb2)

    def test_playlist_changed(self):
        for i in range(1, 11):
            self.add_track(i)
            self.assertEqual(view.Runtime.get_text(), f" {self.plist.runtime()} ")

            view.switch(curds.playlist.Starred)
            self.assertEqual(view.Runtime.get_text(), "")

            view.switch(self.plist)
            self.assertEqual(view.Runtime.get_text(), f" {self.plist.runtime()} ")

    def test_stream_start(self):
        curds.playlist.select(self.plist)

        for i in range(1, 11):
            self.add_track(i)

        for i in range(-1, 10):
            self.plist.current = i
            curds.notify.notify("stream-start")
            if i == -1:
                self.assertEqual(view.Selection.count_selected_rows(), 0)
            else:
                self.assertEqual(view.Selection.count_selected_rows(), 1)
                self.assertTrue( view.Selection.path_is_selected(Gtk.TreePath(i)))
            view.Selection.unselect_all()

    def test_scroll_to(self):
        for i in range(1, 11):
            self.add_track(i)

        for i in range(-1, 10):
            view.scroll_to(i)
            if i == -1:
                self.assertEqual(view.Selection.count_selected_rows(), 0)
            else:
                self.assertEqual(view.Selection.count_selected_rows(), 1)
                self.assertTrue( view.Selection.path_is_selected(Gtk.TreePath(i)))

    def test_switch_scroll(self):
        for i in range(1, 11):
            self.add_track(i)

        for i in range(-1, 10):
            self.plist.current = i
            view.switch(self.plist, force=True)
            if i == -1:
                self.assertEqual(view.Selection.count_selected_rows(), 0)
            else:
                self.assertEqual(view.Selection.count_selected_rows(), 1)
                self.assertTrue( view.Selection.path_is_selected(Gtk.TreePath(i)))

    def test_row_activate(self):
        column = view.TreeView.get_column(1)

        for i in range(1, 11):
            self.add_track(i)

        for i in range(10):
            path = Gtk.TreePath(i)
            view.TreeView.row_activated(path, column)

            self.assertEqual(self.plist.current, i)
            self.assertEqual(audio.playbin.get_state(),    Gst.State.PLAYING)
            self.assertEqual(audio.playbin.duration(), (i + 1) * Gst.SECOND)

    def test_add_remove_starred(self):
        event   = Gdk.EventButton.new(Gdk.EventType.BUTTON_PRESS)
        event.x = 5
        column  = view.TreeView.get_column(0)

        for i in range(1, 11):
            self.add_track(i)

        for i in range(10):
            area = view.TreeView.get_cell_area(Gtk.TreePath(i), column)
            event.y = area.y + 5

            event.x = area.x + area.width + 5
            view.TreeView.emit("button-press-event", event)
            self.assertNotIn(self.plist[i], curds.playlist.Starred.list)

            event.x = area.x + 5
            view.TreeView.emit("button-press-event", event)
            self.assertIn(self.plist[i], curds.playlist.Starred.list)
            view.TreeView.emit("button-press-event", event)
            self.assertNotIn(self.plist[i], curds.playlist.Starred.list)

        view.Selection.select_all()
        view.TreeView.emit("button-press-event", event)
        for track in self.plist.list:
            self.assertIn(track, curds.playlist.Starred.list)
        view.TreeView.emit("button-press-event", event)
        for track in self.plist.list:
            self.assertNotIn(track, curds.playlist.Starred.list)

    def test_add_remove_up_next(self):
        event   = Gdk.EventButton.new(Gdk.EventType.BUTTON_PRESS)
        event.x = 5
        column  = view.TreeView.get_column(1)

        for i in range(1, 11):
            self.add_track(i)

        for i in range(10):
            curds.playlist.select(curds.playlist.Starred)
            area = view.TreeView.get_cell_area(Gtk.TreePath(i), column)
            event.y = area.y + 5

            event.x = area.x + area.width + 5
            view.TreeView.emit("button-press-event", event)
            self.assertNotIn(self.plist[i], curds.playlist.UpNext.list)
            self.assertNotEqual(curds.playlist.Current[0], curds.playlist.UpNext)

            event.x = area.x + 5
            view.TreeView.emit("button-press-event", event)
            self.assertIn(self.plist[i], curds.playlist.UpNext.list)
            self.assertEqual(curds.playlist.Current[0], curds.playlist.UpNext)
            view.TreeView.emit("button-press-event", event)
            self.assertNotIn(self.plist[i], curds.playlist.UpNext.list)

        view.Selection.select_all()
        view.TreeView.emit("button-press-event", event)
        for track in self.plist.list:
            self.assertIn(track, curds.playlist.UpNext.list)
        view.TreeView.emit("button-press-event", event)
        for track in self.plist.list:
            self.assertNotIn(track, curds.playlist.UpNext.list)

    def test_filter_error(self):
        view.Entry.set_text("*v")
        view.Entry.emit("search-changed")

        self.assertTrue(view.Entry.get_style_context().has_class("warning"))
        self.assertTrue(view.Popover.is_visible())
        self.assertEqual(view.Error.get_text(), "Nothing to repeat at position 0")

        view.Entry.set_text("")
        view.Entry.emit("search-changed")
        gtk.main_loop(iteration_delay=0.01)

        self.assertFalse(view.Entry.get_style_context().has_class("warning"))
        self.assertFalse(view.Popover.is_visible())

    def test_filter_tracks(self):
        for i in range(1, 11):
            self.add_track(i)
        self.assertEqual(view.Filter.iter_n_children(None), 10)

        view.Entry.set_text("Test Track 01")
        view.Entry.emit("search-changed")
        self.assertEqual(view.Filter.iter_n_children(None), 1)

        view.Entry.set_text("")
        view.Entry.emit("search-changed")
        self.assertEqual(view.Filter.iter_n_children(None), 10)
